@@ -95,5 +95,8 @@ netk_tasklist_get_type (void)
                                          NetkLoadIconFunction load_icon_func,
                                          void *data,
                                          GDestroyNotify free_data_func);
+
+     void netk_tasklist_set_button_relief (NetkTasklist * tasklist,
+                                           GtkReliefStyle relief);
 G_END_DECLS
 #endif /* NETK_TASKLIST_H */
