@@ -71,6 +71,7 @@ struct _NetkWindowPrivate
     NetkClassGroup *class_group;
     Window group_leader;
     Window transient_for;
+    GdkRectangle icon_geometry;
     char *name;
     char *icon_name;
     char *session_id;
@@ -186,31 +187,20 @@ static guint signals[LAST_SIGNAL] = { 0 };
 GType
 netk_window_get_type (void)
 {
-    static GType object_type = 0;
+    static GType type = G_TYPE_INVALID;
 
-    g_type_init ();
-
-    if (!object_type)
+    if (G_UNLIKELY (type == G_TYPE_INVALID))
     {
-        static const GTypeInfo object_info = {
-            sizeof (NetkWindowClass),
-            (GBaseInitFunc) NULL,
-            (GBaseFinalizeFunc) NULL,
-            (GClassInitFunc) netk_window_class_init,
-            NULL,               /* class_finalize */
-            NULL,               /* class_data */
-            sizeof (NetkWindow),
-            0,                  /* n_preallocs */
-            (GInstanceInitFunc) netk_window_init,
-            NULL                /* value_table */
-        };
-
-        object_type =
-            g_type_register_static (G_TYPE_OBJECT, "NetkWindow", &object_info,
-                                    0);
+        type = _netk_g_type_register_simple (G_TYPE_OBJECT,
+                                             "NetkWindow",
+                                             sizeof (NetkWindowClass),
+                                             netk_window_class_init,
+                                             sizeof (NetkWindow),
+                                             netk_window_init,
+                                             0);
     }
 
-    return object_type;
+    return type;
 }
 
 static void
@@ -222,6 +212,8 @@ netk_window_init (NetkWindow * window)
     window->priv->icon_name = NULL;
     window->priv->client_machine = NULL;
     window->priv->workspace = ALL_WORKSPACES;
+    
+    window->priv->icon_geometry.width = -1;
 
     window->priv->icon_cache = p_netk_icon_cache_new ();
 }
@@ -238,13 +230,13 @@ netk_window_class_init (NetkWindowClass * klass)
     object_class->finalize = netk_window_finalize;
 
     signals[NAME_CHANGED] =
-        g_signal_new ("name_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("name_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           name_changed), NULL,
                       NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[STATE_CHANGED] =
-        g_signal_new ("state_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("state_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           state_changed),
                       NULL, NULL, p_netk_marshal_VOID__FLAGS_FLAGS,
@@ -252,19 +244,19 @@ netk_window_class_init (NetkWindowClass * klass)
                       NETK_TYPE_WINDOW_STATE);
 
     signals[WORKSPACE_CHANGED] =
-        g_signal_new ("workspace_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("workspace_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           workspace_changed),
                       NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[ICON_CHANGED] =
-        g_signal_new ("icon_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("icon_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           icon_changed), NULL,
                       NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
     signals[ACTIONS_CHANGED] =
-        g_signal_new ("actions_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("actions_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           actions_changed),
                       NULL, NULL, p_netk_marshal_VOID__FLAGS_FLAGS,
@@ -272,7 +264,7 @@ netk_window_class_init (NetkWindowClass * klass)
                       NETK_TYPE_WINDOW_ACTIONS);
 
     signals[GEOMETRY_CHANGED] =
-        g_signal_new ("geometry_changed", G_OBJECT_CLASS_TYPE (object_class),
+        g_signal_new (I_("geometry_changed"), G_OBJECT_CLASS_TYPE (object_class),
                       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (NetkWindowClass,
                                                           geometry_changed),
                       NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
@@ -1294,6 +1286,17 @@ void
 netk_window_set_icon_geometry (NetkWindow * window, int x, int y, int width,
                                int height)
 {
+    if ((window->priv->icon_geometry.x == x)  &&
+        (window->priv->icon_geometry.y == y) &&
+        (window->priv->icon_geometry.width == width) &&
+        (window->priv->icon_geometry.height == height))
+            return;
+
+    window->priv->icon_geometry.x = x;
+    window->priv->icon_geometry.y = y;
+    window->priv->icon_geometry.width = width;
+    window->priv->icon_geometry.height = height;
+
     p_netk_set_icon_geometry (window->priv->xwindow, x, y, width, height);
 }
 
