@@ -1,8 +1,8 @@
-/* 
+/*
  * xfce-appmenuitem - a GtkImageMenuItem subclass that handles menu
  * items that are intended to represent applications
  *
- * Copyright (c) 2004 Brian Tarricone <bjt23@cornell.edu>
+ * Copyright (c) 2004-2007 Brian Tarricone <bjt23@cornell.edu>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,6 +45,9 @@ struct _XfceAppMenuItemPriv {
     gchar *icon_name;
     gchar *icon_path;
     gboolean icon_set;
+    
+    gchar *command_expanded;
+    gchar *dot_desktop_filename;
 };
 
 enum {
@@ -86,9 +89,93 @@ _style_set_cb(GtkWidget *w, GtkStyle *prev_style, gpointer user_data)
     style = gtk_rc_get_style_by_paths(gtk_settings_get_default(),
             "GtkMenuItem", "GtkMenuItem", GTK_TYPE_IMAGE_MENU_ITEM);
     children = gtk_container_get_children(GTK_CONTAINER(w));
-    for(l = children; l; l = l->next)
+    for(l = children; l; l = l->next) {
         if(GTK_IS_WIDGET(l->data))
-            gtk_widget_set_style(GTK_WIDGET(l->data), style);    
+            gtk_widget_set_style(GTK_WIDGET(l->data), style);
+    }
+    g_list_free(children);
+}
+
+static void
+_expand_percent_vars(XfceAppMenuItem *app_menu_item)
+{
+    GString *newstr;
+    gchar *p;
+    
+    g_return_if_fail(app_menu_item->priv->command);
+    
+    newstr = g_string_sized_new(strlen(app_menu_item->priv->command) + 1);
+    
+    for(p = app_menu_item->priv->command; *p; ++p) {
+        if('%' == *p) {
+            ++p;
+            switch(*p) {
+                /* we don't care about these since we aren't passing filenames */
+                case 'f':
+                case 'F':
+                case 'u':
+                case 'U':
+                /* these are all deprecated */
+                case 'd':
+                case 'D':
+                case 'n':
+                case 'N':
+                case 'v':
+                case 'm':
+                    break;
+                
+                case 'i':
+                    if(G_LIKELY(app_menu_item->priv->icon_name)) {
+                        gchar *str = g_shell_quote(app_menu_item->priv->icon_name);
+                        g_string_append(newstr, "--icon ");
+                        g_string_append(newstr, str);
+                        g_free(str);
+                    }
+                    break;
+                
+                case 'c':
+                    if(G_LIKELY(app_menu_item->priv->name)) {
+                        gchar *name_locale, *str;
+                        gsize bread = 0;
+                        GError *error = NULL;
+                        
+                        name_locale = g_locale_from_utf8(app_menu_item->priv->name,
+                                                         -1, &bread, NULL,
+                                                         &error);
+                        if(name_locale) {
+                            str = g_shell_quote(name_locale);
+                            g_string_append(newstr, str);
+                            g_free(str);
+                            g_free(name_locale);
+                        } else {
+                            g_warning("Invalid UTF-8 in Name at byte %d: %s",
+                                      bread, error->message);
+                        }
+                    }
+                    break;
+                
+                case 'k':
+                    if(app_menu_item->priv->dot_desktop_filename) {
+                        gchar *str = g_shell_quote(app_menu_item->priv->dot_desktop_filename);
+                        g_string_append(newstr, str);
+                        g_free(str);
+                    }
+                    break;
+                
+                case '%':
+                    g_string_append_c(newstr, '%');
+                    break;
+                
+                default:
+                    g_warning("Invalid field code in Exec line: %%%c", *p);
+                    break;
+            }
+        } else
+            g_string_append_c(newstr, *p);
+    }
+        
+    app_menu_item->priv->command_expanded = newstr->str;
+    g_string_free(newstr, FALSE);
 }
 
 static void
@@ -99,12 +186,23 @@ _command_activate_cb(GtkMenuItem *menu_item, gpointer user_data)
     
     g_return_if_fail(app_menu_item && app_menu_item->priv->command);
     
+    /* we do this here instead of in _new*() for 2 reasons:
+     *   1. menu items that never get activated won't slow us down for no
+     *      reason.
+     *   2. we can't guarantee that the icon name or whatever (which can
+     *      influence the result of _expand_percent_vars()) has been set
+     *      when the command is first set.
+     */
+    if(!app_menu_item->priv->command_expanded)
+        _expand_percent_vars(app_menu_item);
+    
     gscreen = gtk_widget_get_screen(GTK_WIDGET(menu_item));
-    if(!xfce_exec_on_screen(gscreen, app_menu_item->priv->command,    app_menu_item->priv->needs_term,
-            app_menu_item->priv->snotify, NULL))
+    if(!xfce_exec_on_screen(gscreen, app_menu_item->priv->command_expanded,
+                            app_menu_item->priv->needs_term,
+                            app_menu_item->priv->snotify, NULL))
     {
         g_warning("XfceAppMenuItem: unable to spawn %s\n",
-                app_menu_item->priv->command);
+                  app_menu_item->priv->command_expanded);
     }
 }
 
@@ -252,6 +350,8 @@ xfce_app_menu_item_finalize(GObject *object)
     g_free(app_menu_item->priv->command);
     g_free(app_menu_item->priv->icon_name);
     g_free(app_menu_item->priv->icon_path);
+    g_free(app_menu_item->priv->command_expanded);
+    g_free(app_menu_item->priv->dot_desktop_filename);
     
     G_OBJECT_CLASS(xfce_app_menu_item_parent_class)->finalize(object);
 }
@@ -435,7 +535,8 @@ xfce_app_menu_item_new_from_desktop_entry(XfceDesktopEntry *entry,
     XfceAppMenuItem *app_menu_item;
     GtkWidget *accel_label;
     gchar *name = NULL, *cmd = NULL, *icon = NULL, *snotify = NULL;
-    gchar *onlyshowin = NULL, *categories = NULL, *term = NULL, *p;
+    gchar *onlyshowin = NULL, *categories = NULL, *term = NULL;
+    const gchar *dfile;
     
     g_return_val_if_fail(XFCE_IS_DESKTOP_ENTRY(entry), NULL);
     
@@ -520,12 +621,6 @@ xfce_app_menu_item_new_from_desktop_entry(XfceDesktopEntry *entry,
         gtk_widget_destroy(GTK_WIDGET(app_menu_item));
         return NULL;
     }
-    
-    /* naive approach to sanitising these */
-    if((p=strchr(cmd, '%'))) {
-        if((p=strchr(cmd, ' ')))
-            *p = 0;
-    }    
 
     /* remove quotes around the command (yes, people do that!) */
     if(cmd[0] == '"') {
@@ -552,6 +647,10 @@ xfce_app_menu_item_new_from_desktop_entry(XfceDesktopEntry *entry,
         }
     }
     
+    dfile = xfce_desktop_entry_get_file(entry);
+    if(dfile)
+        app_menu_item->priv->dot_desktop_filename = g_strdup(dfile);
+    
     return GTK_WIDGET(app_menu_item);
 }
 
